class SchemazerConfig:
    QUERY_TOKEN = 'token'
    HEADER_TOKEN = 'Authentication-Token'
    COOKIE_TOKEN = 'auth_token'

    QUERY_APIKEY = 'apikey'
    HEADER_APIKEY = 'Apikey'
    COOKIE_APIKEY = 'apikey'

    ERROR_HTTP_CODE = 200
    HOST = 'localhost'
    HTTP_SCHEMA = 'http'
    VERSION = 'schemazer'

    PROJECT_NAME = 'Unknown'
    DOCS_PATH = 'documentation'
