ERROR_WRONG = 'SchemazerParameter `{0}` contains non-valid characters.'
ERROR_TYPE = 'SchemazerParameter `{0}` have invalid type.'
ERROR_MISSING = 'Required parameter `{0}` missing.'
ERROR_INTERVAL = 'SchemazerParameter `{0}` out of available values.'
