{% extends "frame.html" %}
{% block content %}

<div class="block">
    <a href="/{{docs_path}}/methods/{{group}}">{{group}}</a>  <  <b>{{method}}</b>
    <p>
        {{method_schema.description}}
    </p>
    <div class="logohint">{{method_schema.url}}</div>

</div>

{% if method_schema.parameters %}
<div class="block">
    <h3>Query parameters</h3>
    <p>
        Required parameters mark of <span style="background-color: #f6fefb">color</span>.
    </p>

    <table class="docutils" border="1">
        <colgroup>
            <col width="25%">
            <col width="75%">
        </colgroup>

        <tbody valign="top">
            {% for param in method_schema.parameters %}
            <tr class="row-even">
                <td title="{% if param.required %}Required{% endif %}">{% if param.required %}* {% endif %}<b>{{param.name}}</b> ({{param.type.__name__}})</td>
                <td>
                    {{param.description_ru}}
                    <br/>
                    <em>{{param.example}}</em>
                </td>
            </tr>
            {% endfor %}
        </tbody>
    </table>

</div>
{% endif %}

<div class="block">
    <em>Response schema</em>
    <pre class="code">{{response.schema}}</pre>
</div>

<div class="block">
    <em>Response example</em>
    <pre class="code">{{response.example}}</pre>
</div>

<div class="block">
    <em>Errors codes</em>
    <p>All errors in page <a href="/{{docs_path}}/errors">errors</a></p>
    <ul>
    {% for error in errors %}
        <li><a href="/{{docs_path}}/errors/{{error.code}}">{{error.code}}</a></li>
    {% endfor %}
    </ul>
</div>
<br>
<br>
<br>
{% endblock content %}